/*
 * distromatic - tool for RPM based repositories
 *
 * Copyright (C) 2004-2021 by Silvan Calarco <silvan.calarco@mambasoft.it>
 * Copyright (C) 2006 by Davide Madrisan <davide.madrisan@gmail.com>
 */

#ifndef CHANGELOG_H
#define CHANGELOG_H

#if TIME_WITH_SYS_TIME
# include <sys/time.h>
# include <time.h>
#else
# if HAVE_SYS_TIME_H
#  include <sys/time.h>
# else
#  include <time.h>
# endif
#endif

#include "distromatic.h"
#include "config.h"
#include "rpmfunctions.h"

struct changeLog {
     long time;
     struct Packager *pkg;
     char *release;
     char *text;
     struct changeLog *next;
     long id;
};

struct changeLogFull {
    struct headerSourceList *source;
    struct changeLog *changelog;
    struct changeLogFull *next;
};

struct Packager {
     char *name;
     char *alias[PACKAGER_MAXALIASES];
     int role;
     int changes_count;
     int packages_count;
     struct Packager *next;
     long id;
};

struct Packager *firstPackager(void);

long changelogTotal(void);

int printChangelog(struct changeLog *changelog);

int printChangelogSince(
     FILE *fout,
     struct configTag *ct,
//     struct headerSourceList *headersourcelist,
     struct tm *sincetime,
     int html);

struct Packager *
getPackagerByName(char *name, int create);

struct changeLog *
getPackageChangelog(uint_32 *changelogtime, char **changelogrelease,
   char **changelogtext, int count, struct headerSourceList* headersource);

char *changeLogTimeStr(char buf[16], struct changeLog *changelog);

#endif                          /* FUNCTIONS_H */
