/*
 * distromatic - tool for RPM based repositories
 *
 * Copyright (C) 2004-2020 by Silvan Calarco <silvan.calarco@mambasoft.it>
 * Copyright (C) 2006 by Davide Madrisan <davide.madrisan@gmail.com>
 */

#ifndef HEADERLIST_H
#define HEADERLIST_H

#define PROVIDEDLIST_IDX_SIZE 20

#ifndef RPMFUNCTIONS_H
#include "rpmfunctions.h"
#endif

#include "distromatic.h"

struct providedList {
     char *name;
     long flags;
     int numproviders;
     struct headerList **provider;
     int numbuildproviders;
     struct headerList **buildprovider;
     char **version;
     struct providedList *next;
     long id;
};

struct fileTree {
     char *name;
     struct fileTree *firstchild;
     struct fileTree *next;
     struct fileTree *parent;
     int numproviders;
     struct headerList **provider;
     long id;
};

struct fileUserList {
     char *name;
     struct fileUserList* next;
     long id;
};

struct fileGroupList {
     char *name;
     struct fileGroupList* next;
     long id;
};

struct warningList {
     char* text;
     struct warningList *next;
};

struct rebuildList {
     struct headerSourceList *sourceheader;
     struct headerList *provider;
     struct rebuildList *next;
};

struct Require {
     char *name;
     long flags;
     char *version;
     struct providedList *resolved;
};

struct headerList {
     char *name;
     int altrepository;
     long epoch;
     char *version;
     char *release;
     char *summary;
     char *group;
     char *description;
     char *sourcename;
     char *arch;
     long size;
     char **obsoletename;
     uint_32 *obsoleteflags;
     char **obsoleteversion;
     int obsoletecount;
     char **providename;
     uint_32 *provideflags;
     char **provideversion;
     int providecount;
     int requirecount;
     struct Require **require;
     struct providedList **provided;
     int filenamecount;
     struct fileTree **file;
     int_16 *fileflags;
     struct fileUserList **fileuser;
     struct fileGroupList **filegroup;
     int recursed;
     struct requireList *requirelist;
     struct headerList *next, *nextbrother;
     struct headerSourceList *sourceheader;
     int obsoleted;
     long id;
};

struct headerSourceList {
     char *name;
     int altrepository;
     long epoch;
     char *version;
     char *release;
     char *summary;
     struct Packager *packager;
     char *group;
     char *description;
     char *url;
     char *license;
     char **source;
     char **patch;
     char *arch;
     char *buildarchs;
     char *excludearch;
     int *dirindex;
     long size;
     char **dirname;
     char **basename;
     int filenamecount;
     long buildtime;
     int requirecount;
     struct Require **require;
     struct providedList **required;
     //struct providedList **provided;
     int providecount;
     struct requireList *requirelist;
     struct headerSourceList *next;
     struct headerSourceList *old;
     struct changeLog *changelog;
     struct headerList *firstchild[ARCHS_MAX];
     struct warningList *firstwarning;
     struct headerSourceList *updatingparent;
     struct rebuildList *firstrebuild;
     long childrenrequiredcount[ARCHS_MAX];
     long id;
};

struct headerStats {
     int headercount[ARCHS_MAX], headersourcecount;
     long long headersize[ARCHS_MAX], headersourcesize;
     struct headerSourceList **headersourcelistvec;
     struct headerList **headerlistvec[ARCHS_MAX];
     long changelog_total;
};

struct headerSourceListIndex {
     struct headerSourceList *headersourcelist;
};

struct headerList *findPackageByName(struct headerList *list, char *name);

struct headerSourceList *findSourcePackage(
     struct headerSourceList *list,
     char *name, char *version, char *release, int altrepository);

struct configTag;

int getPackageInfoIntoHeaderList(Header h, struct headerList *hl);

int getPackageInfoIntoHeaderSourceList(
     Header h, struct headerSourceList *hl);

int compareSourceHeaderBuildDate(const void *prt1, const void *prt2);

int compareGroup(const void *ptr1, const void *ptr2);

int generateHeaderStats(
     struct configTag *ct,
     int arch);

int
generateHeaderSourceStats(struct configTag *ct);

struct providedList*
findOrCreateProvidedListEntry(struct providedList **idx,
                              char* findname, int create, int arch);

struct fileTree*
findOrCreateFileTreeBrother(struct fileTree* *first,char* findname, int arch);

struct fileTree*
findOrCreateFileTreeEntry(struct fileTree* *first,char* findname, int arch);

void
cleanProvidedListIndex(struct configTag *ct, int arch);

void
createProvidedListIndex(struct providedList* *idx, int arch);

int generateHeaderList(struct configTag *ct, int arch, int incremental);

int generateSourceHeaderList(struct configTag *ct, int mode, int incremental);

/*int generateSourceHeaderList(
     struct headerSourceList **headersourcelist,
     const char *scanpath, const char **altscanpath, int mode);*/

int
addToSourceHeaderList(
     struct configTag *ct,
     int mode,
     int altrepository);

int
addToHeaderList(struct configTag *ct,
                const char *scanpath, /*int mode,*/
                const char* scantag,
                const char* altscanpath[ALT_REPS_MAX],
                const char* altscantag[ALT_REPS_MAX],
                int arch);

struct warningList* addWarning(struct headerSourceList *pkg,char* text);

struct rebuildList* addRebuild(struct headerSourceList *pkg, struct headerSourceList* sourceheader, struct headerList* provider);

int getPackageNameFromFile(char *name);

#endif                          /* HEADERLIST_H */
