/*
 * distromatic - tool for RPM based repositories
 *
 * Copyright (C) 2004-2020 by Silvan Calarco <silvan.calarco@mambasoft.it>
 * Copyright (C) 2006 by Davide Madrisan <davide.madrisan@gmail.com>
 */

#ifndef RPMFUNCTIONS_H
#define RPMFUNCTIONS_H

/* Define to the RPM version */
#define RPM_VERSION ((RPM_VERSION_MAJOR<<16)|(RPM_VERSION_MINOR<<8)|RPM_VERSION_MICRO)

#if RPM_VERSION_MAJOR == 4
#define uint_32 uint32_t
#define int_16 rpmFlags
#define HE_t rpmTagVal
#endif

#ifndef H_RPMLIB
# include <rpm/rpmlib.h>
#endif

#if RPM_VERSION >= 0x040100
#include <rpm/rpmts.h>
#else
#define rpmReadPackageFile(a,b,c,d) rpmReadPackageHeader(b,d,0,NULL,NULL)
#endif

#if RPM_VERSION >= 0x050000
#ifndef H_RPMEVR
#include <rpm/rpmevr.h>
#endif
#endif

#include <dirent.h>

#define HEADERS_BUFFER_SIZE 2000000

unsigned int checkVersionWithFlags(const char* cmp1, const uint_32 flags, const char* cmp2);

void rpminit(void);

char *headerGetStringEntry(Header h, const int tag);

char **headerGetStringArrayEntry(Header h, const int tag, int* count);

long long headerGetUIntEntry(Header h, const int tag);

uint_32 *headerGetUIntArrayEntry(Header h, const int tag, int *count);

int getHeader(rpmts *ts, char *headerFile, Header * h);

int getPackageRequires(
     Header h, char ***requirename, uint_32 **requireflags,
     char ***requireversion, int *requirecount);

int getPackageProvides(
     Header h, char ***providename, uint_32 **provideflags,
     char ***provideversion, int *providecount);

int getPackageObsoletes(
     Header h, char ***obsoletename, uint_32 **obsoleteflags,
     char ***obsoleteversion, int *obsoletecount);

int
getPackageFiles(Header h, uint_32 **dirindexes,
                char ***dirnames, int *dirnamescount,
                char ***basenames, int *filenamescount,
                char ***usernames, char ***groupnames,
                int_16 **fileflags);

int rpmnameidx(char *filename);

int rpmnamecmp(char *filename1, char *filename2, int checkver);

int scanrpmnamecmp(const struct dirent **f1, const struct dirent **f2);

char* printrpmversion(char *s, int bufsize, long epoch, char *version, char *release);

#endif                          // RPMFUNCTIONS_H
